package org.projectopen.timesheet;

/*
 * Copyright (C) 2010 ]project-open[ 
 *
 * This program is free software. You can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option)
 * any later version. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * @author Frank Bergmann (frank.bergmann@project-open.com)
 * 
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.table.TableColumn;
import org.projectopen.treetable.JTreeTable;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URI;
import java.util.Properties;

@SuppressWarnings("deprecation")

/*
 * Integration between the TrayIcon, the configuration frame and
 * the timesheet frame:
 * 		- Setup the PopUp Menu of the TrayIcon
 * 		- Setup the Tooltip
 * 		- Register listeners that will start the configuraiton and timesheet frames
 */
public class SystemTrayTest
{
    static TrayIcon trayIcon = null;
    
	public static JFrame treeFrame;
    public static JFrame configFrame;
    
    static int mouseX, mouseY;
    
    static String server = "http://po34demo.project-open.net";
    static String port = "";
    static String email = "bbigboss@tigerpond.com";
    static String password = "ben";
    static Properties properties = null;

    
    public static Properties getProperties() {
    	
    	if (properties == null) {

			// create application properties with default
			properties = new Properties();
			properties.put("server", server);
			properties.put("port", port);
			properties.put("email", email);
			properties.put("password", password);

			try {
    			// now load properties from last invocation
				FileInputStream in = new FileInputStream("properties");
    			properties.load(in);
    			in.close();

    		} catch (Exception e) {
    			// ignore if not there
    			// e.printStackTrace();
    		}
    	}

		return properties;
	}

    public static void storeProperties() {
    	FileOutputStream out;
		try {
			out = new FileOutputStream("properties");
	    	properties.store(out, "--- User account to access ]project-open[ server ---");
	    	out.close();
		} catch (Exception e) {
			// ignore errors
			// e.printStackTrace();
		}
    }
    
    
	//Optionally set the look and feel.
    private static boolean useSystemLookAndFeel = true;

    public static void trayIconError(TrayIcon.MessageType errorType, String title, String message) {
    	TrayIcon trayIcon = SystemTrayTest.getTrayIcon();
    	trayIcon.displayMessage(title, message, errorType);
    }	
	
    
    public static TrayIcon getTrayIcon() {
		return trayIcon;
	}
    
    public static String getServer() {
    	return server;
    }
    
    public static String getEmail() {
    	return email;
    }
    
    public static String getPassword() {
    	return password;
    }
    
    
    // Show timesheet logging frame
	private void launchHoursPanel() {

    	System.out.println("SystemTrayTest.launchHoursPanel");
		
    	//Create and set up the window.
    	if (treeFrame == null) {
    		treeFrame = new JFrame("TreeTable");

        	System.out.println("SystemTrayTest.launchHoursPanel: before new model");

    		ProjectTreeTableModel model = new ProjectTreeTableModel();
    		JTreeTable treeTable = new JTreeTable(model);

    		// show number in US-American locale "12.34", as it is
    		// standard in ]po[.
    		treeTable.setDefaultRenderer(Float.class, new NumberRenderer());
    		treeTable.setDefaultRenderer(Double.class, new NumberRenderer());
    		treeTable.setDefaultRenderer(Number.class, new NumberRenderer());
    		
    		// set the column width
    		int colWidth[] = {300, 50, 200};
    		TableColumn column = null;
    		for (int i = 0; i < 3; i++) {
    		    column = treeTable.getColumnModel().getColumn(i);
    		    column.setPreferredWidth(colWidth[i]);
    		}
    		
    		// set alignment of Hours column
		    column = treeTable.getColumnModel().getColumn(1);
    		
    		treeFrame.getContentPane().add(new JScrollPane(treeTable));
    		treeFrame.pack();

    		// lower right corner of the frame => mouse position
    		Dimension dim = treeFrame.getSize();
    		treeFrame.setLocation(mouseX - dim.width, mouseY-dim.height);

    		// show the frame
    		treeFrame.show();

    	}
        treeFrame.setVisible(true);
    };
    
    
    // Show time sheet logging frame
    private void launchConfigPanel(boolean visible) {
        if (configFrame == null) {
        	configFrame = new JFrame("Configuration");
            configFrame.add(new ConfigPanel(this));
            configFrame.pack();
            Dimension dim = configFrame.getSize();
            configFrame.setLocation(mouseX - dim.width, mouseY-dim.height);
        }
        configFrame.setVisible(visible);    
    };
        
    
    public SystemTrayTest()
    {

        if (SystemTray.isSupported()) {

            SystemTray tray = SystemTray.getSystemTray();
            Image image = Toolkit.getDefaultToolkit().getImage("po-icon.gif");

            MouseListener mouseListener = new MouseListener() {
                
                public void mouseClicked(MouseEvent e) {
                    // System.out.println("Tray Icon - Mouse clicked!" + e.getButton());   
                	
                	// remember the mouse position
                	mouseX = e.getX();
                	mouseY = e.getY();
                    // Take action depending on the button clicked
                    switch (e.getButton()) {
                		case 1: {
                            //Create and set up the window.
                			launchHoursPanel();
                		}
                	}
                }
                public void mouseEntered(MouseEvent e) {
                    // System.out.println("Tray Icon - Mouse entered!");                 
                }
                public void mouseExited(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse exited!");                 
                }
                public void mousePressed(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse pressed!");                 
                }
                public void mouseReleased(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse released!");                 
                }

            };

            // Exit the Tray application
            ActionListener exitListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    System.out.println("Exiting...");
                    System.exit(0);
                }
            };
            
            // Show configuration frame
            ActionListener configListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                	launchConfigPanel(true);
                }
            };
            
            // Show configuration frame
            ActionListener hoursListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
        			launchHoursPanel();                	
                }
            };
            
            // Login to ]po[
            ActionListener loginListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        URI uri = new java.net.URI(getServer() + "/register/index?email=" + getEmail() + "&pass");
                        java.awt.Desktop.getDesktop().browse(uri);
                    } catch (Exception e1) {
                        System.err.println(e1.getMessage());
                    }
                }
            };
            
            // Build the TrayIcon menu
            PopupMenu popup = new PopupMenu();
            
            MenuItem hoursMenuItem = new MenuItem("Log Hours");
            hoursMenuItem.addActionListener(hoursListener);
            popup.add(hoursMenuItem);

            if( java.awt.Desktop.isDesktopSupported() ) {
                MenuItem loginItem = new MenuItem("Login");
                loginItem.addActionListener(loginListener);
                popup.add(loginItem);
            }
            
            MenuItem defaultItem2 = new MenuItem("Configuration");
            defaultItem2.addActionListener(configListener);
            popup.add(defaultItem2);

            MenuItem defaultItem = new MenuItem("Exit");
            defaultItem.addActionListener(exitListener);
            popup.add(defaultItem);

            trayIcon = new TrayIcon(image, "Tray Demo", popup);

            ActionListener actionListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    trayIcon.displayMessage("Action Event", 
                        "An Action Event Has Been Peformed!",
                        TrayIcon.MessageType.INFO);
                }
            };
            
            trayIcon.setImageAutoSize(true);
            trayIcon.addActionListener(actionListener);
            trayIcon.addMouseListener(mouseListener);

            try {
            	tray.add(trayIcon);
            } catch (AWTException e) {
                System.err.println("TrayIcon could not be added.");
            }

        } else {
            System.err.println("System tray is currently not supported.");
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {    	
    	// Use Look and Feel?
        if (useSystemLookAndFeel) {
            try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception ex) {
                System.err.println("Couldn't use system look and feel.");
            }
        }
        
        Properties props = getProperties();
        System.out.println(props);
        storeProperties();
        
        // Start the main application
        SystemTrayTest main = new SystemTrayTest();
        System.out.println(main);
    }
    
}


